'''
This program counts the number of permutations g over S = {0,1,2,...,n-1}
with exactly three fixed points. Permutations are encoded by propositional
variables p(i,j) in the following way:
  g(i) = j iff p(i,j) = 1.
'''
from logic import bool
from logic.abc import p, i, j, k, l
from logic.fo.quantifiers import *
from logic.cl.runner2 import GPURunner

n = 6

# Definition of set S  
S = range(n)
 
# Definition of set D of ordered pairs (a,b) in SxS so that a < b
D = comb(S,2)

# Statement: domain of g is S
f1 = A[i:S].E[j:S] (p(i,j))

# Statement: g is a function
f2 = A[i:S].A[k,l:D] (~p(i,k) | ~p(i,l))

# Statement: g is 1-1
f3 = A[k:S].A[i,j:D] (~p(i,k) | ~p(j,k))

# Definition of set V of ordered quadruplets (a,b,c,d) in S^4
# where a, b, c, and d are mutually distinct elements
V = perm(S,4)

V = (c + (cc,) for c in comb(S, 3) for cc in set(S).difference(c))

# Statement: g has exactly three fixed points
f4 = E[i,j:V].E[k].A[l] (p(i,i) & p(j,j) & p(k,k) & ~p(l,l))

# Result
print(GPURunner().count(bool.Expression(f1 & f2 & f3 & f4)))
