'''
Printer Module

@author: Aleksandar
'''
from logic.bool.algebra import Not, And, Or, Xor, Imp, Eq
from logic.core.ast import Transform


class Printer(Transform):
    '''
    Creates a string representation of a term's AST.
    '''
    
    def __call__(self, term, symbols, *args, **kwargs):
        return super().__call__(term, symbols, *args, **kwargs)
    
    def visit_UnaryOperator(self, term, symbols, *args, **kwargs):
        op = self.default_visit(term, symbols, *args, **kwargs)
        return ''.join(['(', symbols[type(term)], ' ', op[0], ')'])
    
    def visit_BinaryOperator(self, term, symbols, *args, **kwargs):
        op = self.default_visit(term, symbols, *args, **kwargs)
        return ''.join(['(', op[0], ' ', symbols[type(term)], ' ', op[1], ')'])
    
    def visit_Var(self, term, *args, **kwargs):
        return str(term.name) + ''.join(str(i) for i in term.indices)
    
    def visit_Const(self, term, *args, **kwargs):
        return str(term.value)
    
    def default_visit(self, node, *args, **kwargs):
        # TODO: reconsider default visit for transforms
        # without this change printer can't work because
        # we can't negate string (~string), something that
        # is introduced in operator's __new__ method 
        return [self.visit(child, *args, **kwargs) for child in node]


math_symbols = {Not : '¬',
                And : '∧',
                Or : '∨',
                Xor : '⊻',
                Imp : '⇒',
                Eq : '⇔'}


bitwise_symbols = {Not : '~',
                   And : '&',
                   Or : '|',
                   Xor: '^'}


boolean_symbols = {Not : 'not',
                   And : 'and',
                   Or : 'or'}
