'''
Transforms Module

@author: Aleksandar
'''
from logic.core.ast import Transform
from logic.bool.algebra import Not, And, Or


class Identity(Transform):
    '''
    Copies the term.
    '''
    
    def visit_NullaryTerm(self, term, *args, **kwargs):
        return term


class ToBitwise(Identity):
    '''
    Eliminates the implication and equivalence operators, so that
    the resulting term can be evaluated using bitwise operations.
    '''
    
    def visit_Imp(self, term):
        imp_op = self.default_visit(term)
        return Or(Not(imp_op[0]), imp_op[1])
    
    def visit_Eq(self, term):
        eq_op = self.default_visit(term)
        return Or(And(*eq_op), Not(Or(*eq_op)))


class ToBoolean(ToBitwise):
    '''
    Eliminates the xor operator too, so that the resulting term
    can be evaluated using logical operations.
    '''
    
    def visit_Xor(self, term):
        xor_op = self.default_visit(term)
        return And(Not(And(*xor_op)), Or(*xor_op))
