'''
Bool Utilities

@author: Aleksandar
'''
import math


def free_vectors(n_vars, var_width=None, generate=False):
    '''
    Generates n_vars free vectors splitted in chunks of var_width size.

    Pure Python implementation.
    '''
    vector_length = 2 ** n_vars
    if not var_width:
        var_width = vector_length
    
    n_const_vars = min(int(round(math.log(var_width, 2))), n_vars)
    var_width = 2 ** n_const_vars
    const_vars = []
    for i in range(n_const_vars):
        const_vars.append(('0' * 2 ** i + '1' * 2 ** i) * 2 ** (n_const_vars - 1 - i))
    const_vars = tuple(int(var, 2) for var in reversed(const_vars))
    
    n_blocks = vector_length // var_width
    if n_blocks == 1:
        return const_vars
    
    vars_ = []
    zeros = 0
    ones = 2 ** var_width - 1
    if generate:
        def fv_generator():
            for i in range(n_blocks):
                var = [ones if b == '1' else zeros for b in '{0:0{1}b}'.format(i, n_vars - n_const_vars)]
                yield tuple(var) + const_vars
        return fv_generator()
    else:
        for i in range(n_blocks):
            var = [ones if b == '1' else zeros for b in '{0:0{1}b}'.format(i, n_vars - n_const_vars)]
            vars_.append(tuple(var) + const_vars)
        return vars_


# ======================================


import numpy as np


def fv(n):
    '''
    Generates n free vectors.
    
    Returns 2-D array with each row representing one vector.
    '''
    free_vectors = np.zeros((n, 2 ** n), dtype=np.int8)
    for i in range(n):
        free_vectors[0:i, 2 ** i:2 ** (i + 1)] = free_vectors[0:i, 0:2 ** i]
        free_vectors[i, 2 ** i:2 ** (i + 1)] = np.ones(2 ** i, dtype=np.int8)
    return free_vectors


def fv32(n):
    '''
    Generates n free vectors packed in 32-bit numbers.
    
    Returns 2-D array with each row representing one vector.
    '''
    free_vectors = np.packbits(fv(n)) # 8-bit packing
    to32bit = 256 ** np.arange(4)[::-1]
    return free_vectors.reshape(-1, 4).dot(to32bit).reshape(n, -1)
