'''
Core Utilities

@author: Aleksandar
'''
from functools import wraps


def run_once(func):
    '''
    Decorates a function so that it can be called only once.
    '''
    @wraps(func)
    def wrapper(*args, **kwargs):
        if not wrapper.has_run:
            wrapper.has_run = True
            return func(*args, **kwargs)
    wrapper.has_run = False
    return wrapper


def preorder(node, visit):
    '''
    Performs a preorder visit of AST.
    '''
    stack = [node]
    while stack:
        parent = stack.pop()
        visit(parent)
        for child in reversed(parent):
            stack.append(child)
